﻿''' <summary>
''' Bysonブロック
''' </summary>
Public MustInherit Class ByBlock

    ''' <summary>
    ''' ネイティブ型バイトサイズ
    ''' </summary>
    Public Const BYTE_TYPE_SIZE = 1     ' バイト
    Public Const CHAR_TYPE_SIZE = 2     ' Unicode
    Public Const SHORT_TYPE_SIZE = 2    ' 短整数
    Public Const INTEGER_TYPE_SIZE = 4  ' 整数
    Public Const LONG_TYPE_SIZE = 8     ' 長整数
    Public Const SINGLE_TYPE_SIZE = 4   ' 単精度浮動小数点
    Public Const DOUBLE_TYPE_SIZE = 8   ' 倍精度浮動小数点

    ''' <summary>
    ''' 構築
    ''' </summary>
    Protected Sub New()

    End Sub

    ''' <summary>
    ''' 参照オブジェクト収集
    ''' </summary>
    ''' <param name="map">参照オブジェクトMAP</param>
    ''' <returns>参照オブジェクトMAP</returns>
    Public Overridable Function CollectReferenceObject(ByVal map As HashSet(Of Object))
        Return map
    End Function

    ''' <summary>
    ''' 値を格納する
    ''' </summary>
    ''' <returns>インスタンス</returns>
    Public MustOverride Function ResolveValue() As Object

    ''' <summary>
    ''' 書込
    ''' </summary>
    ''' <param name="stream">ストリーム</param>
    ''' <param name="map">Bysonマップ</param>
    Public MustOverride Sub Write(ByVal stream As System.IO.Stream, ByVal map As BysonMap)

    ''' <summary>
    ''' 読込
    ''' </summary>
    ''' <param name="stream">ストリーム</param>
    Public MustOverride Sub Read(ByVal stream As System.IO.Stream, ByVal map As BysonMap)

    ''' <summary>
    ''' バイト配列書込
    ''' </summary>
    ''' <param name="stream">ストリーム</param>
    ''' <param name="value">配列</param>
    Public Shared Sub WriteByte(ByVal stream As System.IO.Stream, ByVal value As Byte())
        stream.Write(value, 0, value.Length)
    End Sub

    ''' <summary>
    ''' 文字書込
    ''' </summary>
    ''' <param name="stream">ストリーム</param>
    ''' <param name="value">文字</param>
    Public Shared Sub WriteChar(ByVal stream As System.IO.Stream, ByVal value As Char)
        WriteByte(stream, BitConverter.GetBytes(value))
    End Sub

    ''' <summary>
    ''' 短整数書込
    ''' </summary>
    ''' <param name="stream">ストリーム</param>
    ''' <param name="value">短整数</param>
    Public Shared Sub WriteShort(ByVal stream As System.IO.Stream, ByVal value As Short)
        WriteByte(stream, BitConverter.GetBytes(value))
    End Sub

    ''' <summary>
    ''' 整数書込
    ''' </summary>
    ''' <param name="stream">ストリーム</param>
    ''' <param name="value">整数</param>
    Public Shared Sub WriteInteger(ByVal stream As System.IO.Stream, ByVal value As Integer)
        WriteByte(stream, BitConverter.GetBytes(value))
    End Sub

    ''' <summary>
    ''' 長整数書込
    ''' </summary>
    ''' <param name="stream">ストリーム</param>
    ''' <param name="value">長整数</param>
    Public Shared Sub WriteLong(ByVal stream As System.IO.Stream, ByVal value As Long)
        WriteByte(stream, BitConverter.GetBytes(value))
    End Sub

    ''' <summary>
    ''' 単精度浮動小数点書込
    ''' </summary>
    ''' <param name="stream">ストリーム</param>
    ''' <param name="value">単精度浮動小数点</param>
    Public Shared Sub WriteSingle(ByVal stream As System.IO.Stream, ByVal value As Single)
        WriteByte(stream, BitConverter.GetBytes(value))
    End Sub

    ''' <summary>
    ''' 倍精度浮動小数点書込
    ''' </summary>
    ''' <param name="stream">ストリーム</param>
    ''' <param name="value">倍精度浮動小数点</param>
    Public Shared Sub WriteDouble(ByVal stream As System.IO.Stream, ByVal value As Double)
        WriteByte(stream, BitConverter.GetBytes(value))
    End Sub

    ''' <summary>
    ''' 文字列書込
    ''' </summary>
    ''' <param name="stream">ストリーム</param>
    ''' <param name="value">文字列</param>
    Public Shared Sub WriteString(ByVal stream As System.IO.Stream, ByVal value As String)
        Dim w As Byte() = System.Text.Encoding.UTF8.GetBytes(value)
        WriteByte(stream, BitConverter.GetBytes(w.Length))
        WriteByte(stream, w)
    End Sub

    ''' <summary>
    ''' バイト配列読込
    ''' </summary>
    ''' <param name="stream">ストリーム</param>
    ''' <param name="length">読込長</param>
    ''' <returns>バイト配列</returns>
    Public Shared Function ReadByte(ByVal stream As System.IO.Stream, ByVal length As Integer) As Byte()
        Dim array() = New Byte(length) {}
        stream.Read(array, 0, length)
        Return array
    End Function

    ''' <summary>
    ''' 文字読込
    ''' </summary>
    ''' <param name="stream">ストリーム</param>
    ''' <returns>文字</returns>
    Public Shared Function ReadChar(ByVal stream As System.IO.Stream) As Char
        Return BitConverter.ToChar(ReadByte(stream, CHAR_TYPE_SIZE), 0)
    End Function

    ''' <summary>
    ''' 短整数読込
    ''' </summary>
    ''' <param name="stream">ストリーム</param>
    ''' <returns>短整数</returns>
    Public Shared Function ReadShort(ByVal stream As System.IO.Stream) As Short
        Return BitConverter.ToInt16(ReadByte(stream, SHORT_TYPE_SIZE), 0)
    End Function

    ''' <summary>
    ''' 整数読込
    ''' </summary>
    ''' <param name="stream">ストリーム</param>
    ''' <returns>整数</returns>
    Public Shared Function ReadInteger(ByVal stream As System.IO.Stream) As Integer
        Return BitConverter.ToInt32(ReadByte(stream, INTEGER_TYPE_SIZE), 0)
    End Function

    ''' <summary>
    ''' 長整数読込
    ''' </summary>
    ''' <param name="stream">ストリーム</param>
    ''' <returns>長整数</returns>
    Public Shared Function ReadLong(ByVal stream As System.IO.Stream) As Long
        Return BitConverter.ToInt64(ReadByte(stream, LONG_TYPE_SIZE), 0)
    End Function

    ''' <summary>
    ''' 単精度浮動小数点読込
    ''' </summary>
    ''' <param name="stream">ストリーム</param>
    ''' <returns>単精度浮動小数点</returns>
    Public Shared Function ReadSingle(ByVal stream As System.IO.Stream) As Single
        Return BitConverter.ToSingle(ReadByte(stream, SINGLE_TYPE_SIZE), 0)
    End Function

    ''' <summary>
    ''' 倍精度浮動小数点読込
    ''' </summary>
    ''' <param name="stream">ストリーム</param>
    ''' <returns>倍精度浮動小数点</returns>
    Public Shared Function ReadDouble(ByVal stream As System.IO.Stream) As Double
        Return BitConverter.ToDouble(ReadByte(stream, DOUBLE_TYPE_SIZE), 0)
    End Function

    ''' <summary>
    ''' 文字列読込
    ''' </summary>
    ''' <param name="stream">ストリーム</param>
    ''' <returns>文字列</returns>
    Public Shared Function ReadString(ByVal stream As System.IO.Stream) As String
        Dim w As Integer = ReadInteger(stream)
        Dim ary As Byte() = ReadByte(stream, w)
        Return BysonMap.Trim(System.Text.Encoding.UTF8.GetString(ary))
    End Function

End Class
