﻿''' <summary>
''' Bysonマップ
''' </summary>
Public Class BysonMap

    ' 無効インデックス
    Public Shared INVALID_INDEX As Integer = -1
    ' 無効ID
    Public Shared INVALID_ID As Long = -1L
    ' アセンブリ
    Private _asm As System.Reflection.Assembly
    ' 外部リスナー
    Private _externalListener As ExternalBlockListener = Nothing

    ''' <summary>
    ''' 構築
    ''' </summary>
    Public Sub New(ByVal asm As System.Reflection.Assembly)
        Me._asm = asm
    End Sub

    ''' <summary>
    ''' クラス情報
    ''' </summary>
    Public Class ClassInformation
        Public Name As String       ' クラス名称
        Public ClassType As Type    ' クラスタイプ(cache)
        Public Fields() As System.Reflection.FieldInfo  ' フィールド情報配列(cache)
        Public FieldMap As New Dictionary(Of String, Integer)   ' フィールド情報インデックスMAP(cache)
    End Class

    ''' <summary>
    ''' インスタンス情報
    ''' </summary>
    Public Class InstanceInformation
        Public ClassIndex As Integer    ' クラスインデックス
        'Public InstanceId As Long       ' インスタンスID
        Public Instance As Object       ' インスタンス
    End Class

    ''' <summary>
    ''' クラス情報配列
    ''' </summary>
    Private _classArray As New List(Of ClassInformation)()

    ''' <summary>
    ''' インスタンス情報配列
    ''' </summary>
    Private _instanceArray As New List(Of InstanceInformation)()

    ''' <summary>
    ''' 属性名称配列
    ''' </summary>
    Private _fieldNameArray As New List(Of String)()

    ''' <summary>
    ''' クラス名称-クラスインデックスMAP
    ''' </summary>
    Private _classIndexMap As New Dictionary(Of String, Integer)

    ''' <summary>
    ''' インスタンス-インスタンスインデックスMAP
    ''' </summary>
    Private _instanceIndexMap As New Dictionary(Of Object, Integer)

    ''' <summary>
    ''' 属性名称-属性名称インデックスMAP
    ''' </summary>
    Private _fieldNameIndexMap As New Dictionary(Of String, Integer)

    ''' <summary>
    ''' 外部リスナー登録
    ''' </summary>
    ''' <param name="listener">外部リスナー</param>
    Public Sub SetExternalBlockListener(ByVal listener As ExternalBlockListener)
        Me._externalListener = listener
    End Sub

    ''' <summary>
    ''' 外部リスナー取得
    ''' </summary>
    ''' <returns>外部リスナー</returns>
    Public Function GetExternalBlockListener() As ExternalBlockListener
        Return Me._externalListener
    End Function

    ''' <summary>
    ''' 全オブジェクト取得
    ''' </summary>
    ''' <returns>オブジェクト配列</returns>
    Public Function GetInstances() As System.Collections.Generic.Dictionary(Of Object, Integer).KeyCollection
        Return Me._instanceIndexMap.Keys()
    End Function

    ''' <summary>
    ''' オブジェクトからインスタンスインデックスを得る
    ''' </summary>
    ''' <param name="o">オブジェクト</param>
    ''' <returns>インスタンスインデックス</returns>
    Public Function ToInstanceIndex(ByVal o As Object) As Integer
        Dim ret As Integer = INVALID_INDEX
        If Not IsNothing(o) Then
            ret = GetInstanceIndex(o)
        End If
        Return ret
    End Function

    ''' <summary>
    ''' インスタンスインデックスからオブジェクトを得る
    ''' </summary>
    ''' <param name="index">インスタンスインデックス</param>
    ''' <returns>オブジェクト</returns>
    Public Function ToObject(ByVal index As Integer) As Object
        Dim ret As Object = Nothing
        If index <> INVALID_INDEX Then
            Try
                ret = Me._instanceArray.Item(index).Instance
            Catch ex As Exception
                System.Diagnostics.Debug.Print(">>" + ex.StackTrace())
                Throw ex
            End Try
        End If
        Return ret
    End Function

    ''' <summary>
    ''' クラス情報作成
    ''' </summary>
    ''' <param name="t">型</param>
    ''' <returns>クラス情報</returns>
    Private Function CreateClassInformation(ByVal t As Type) As ClassInformation
        Dim ci As New ClassInformation()
        ci.Name = t.FullName
        ci.ClassType = t
        ci.Fields = GetFieldInformations(t, ci.FieldMap)
        Return ci
    End Function

    ''' <summary>
    ''' 属性情報を取得する
    ''' </summary>
    ''' <param name="t">型</param>
    ''' <returns>属性情報配列</returns>
    Private Function GetFieldInformations(ByVal t As Type, ByVal fieldMap As Dictionary(Of String, Integer)) As System.Reflection.FieldInfo()
        Dim array As New List(Of System.Reflection.FieldInfo)()
        Dim bindingAttr As System.Reflection.BindingFlags = System.Reflection.BindingFlags.Public Or System.Reflection.BindingFlags.Instance Or System.Reflection.BindingFlags.Static Or System.Reflection.BindingFlags.NonPublic
        Do While Not IsNothing(t) AndAlso Not t.Equals(GetType(Object))
            For Each info As System.Reflection.MemberInfo In t.GetMembers(bindingAttr)
                For Each att As Attribute In info.GetCustomAttributes(False)
                    Dim member As System.Runtime.Serialization.DataMemberAttribute = TryCast(att, System.Runtime.Serialization.DataMemberAttribute)
                    If Not IsNothing(member) Then
                        If TypeOf info Is System.Reflection.FieldInfo Then
                            Dim fi = TryCast(info, System.Reflection.FieldInfo)
                            array.Add(fi)
                            fieldMap.Add(Trim(fi.Name), array.Count - 1)
                        End If
                    End If
                Next
            Next
            t = t.BaseType
        Loop
        Return array.ToArray()
    End Function

    ''' <summary>
    ''' クラス情報インデックス取得
    ''' </summary>
    ''' <param name="t">型</param>
    ''' <returns>インデックス</returns>
    Public Function GetClassIndex(ByVal t As Type) As Integer
        Dim index As Integer = INVALID_INDEX
        Dim key As String = t.FullName
        If Not Me._classIndexMap.ContainsKey(key) Then
            Dim ci As ClassInformation = CreateClassInformation(t)
            Me._classArray.Add(ci)
            index = Me._classArray.Count - 1
            Me._classIndexMap.Add(key, index)
        Else
            Me._classIndexMap.TryGetValue(key, index)
        End If
        Return index
    End Function

    ''' <summary>
    ''' クラス情報取得
    ''' </summary>
    ''' <param name="index">クラス情報インデックス</param>
    ''' <returns>クラス情報</returns>
    Private Function GetClassInformation(ByVal index As Integer) As ClassInformation
        Try
            Return Me._classArray.Item(index)
        Catch ex As Exception
            System.Diagnostics.Debug.Print("" + ex.StackTrace())
            Throw ex
        End Try
        'Return Me._classArray.Item(index)
    End Function

    ''' <summary>
    ''' クラス情報取得
    ''' </summary>
    ''' <param name="index">クラス情報インデックス</param>
    ''' <returns>クラス情報</returns>
    Public Function GetClass(ByVal index As Integer) As Type
        Dim ret As Type = Nothing
        If index <> INVALID_INDEX Then
            ret = GetClassInformation(index).ClassType
        End If
        Return ret
    End Function

    ''' <summary>
    ''' インスタンス情報作成
    ''' </summary>
    ''' <param name="o"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function CreateInstanceInformation(ByVal o As Object) As InstanceInformation
        Dim ii As InstanceInformation = Nothing
        If Not IsNothing(o) Then
            Dim t As Type = o.GetType()
            Dim index As Integer = GetClassIndex(t)
            ii = New InstanceInformation()
            ii.ClassIndex = index
            ii.Instance = o
        End If
        Return ii
    End Function

    ''' <summary>
    ''' インスタンス情報インデックス取得
    ''' </summary>
    ''' <param name="o">オブジェクト</param>
    ''' <returns>インデックス</returns>
    Private Function GetInstanceIndex(ByVal o As Object) As Integer
        Dim index As Integer = INVALID_INDEX
        Dim key As Object = o
        If Not Me._instanceIndexMap.ContainsKey(key) Then
            Dim ii As InstanceInformation = CreateInstanceInformation(o)
            If Not IsNothing(ii) Then
                Me._instanceArray.Add(ii)
                index = Me._instanceArray.Count - 1
                Me._instanceIndexMap.Add(key, index)
            End If
        Else
            Me._instanceIndexMap.TryGetValue(key, index)
        End If
        Return index
    End Function

    ''' <summary>
    ''' オブジェクト登録チェック
    ''' </summary>
    ''' <param name="o">オブジェクト</param>
    ''' <returns>true=登録済</returns>
    Public Function IsRegistedInstance(ByVal o As Object) As Boolean
        Return Me._instanceIndexMap.ContainsKey(o)
    End Function

    ''' <summary>
    ''' 属性名称インデックス取得
    ''' </summary>
    ''' <param name="name">属性名称</param>
    ''' <returns>インデックス</returns>
    Private Function GetFieldNameIndex(ByVal name As String) As Integer
        Dim index As Integer = INVALID_INDEX
        Dim key As String = name
        If Not Me._fieldNameIndexMap.ContainsKey(key) Then
            Me._fieldNameArray.Add(name)
            index = Me._fieldNameArray.Count - 1
            Me._fieldNameIndexMap.Add(key, index)
        Else
            Me._fieldNameIndexMap.TryGetValue(key, index)
        End If
        Return index
    End Function

    ''' <summary>
    ''' インスタンスインデックスからクラス情報を取得
    ''' </summary>
    ''' <param name="instanceIndex">インスタンスインデックス</param>
    ''' <returns>クラス情報</returns>
    Private Function GetClassInformationByInstanceIndex(ByVal instanceIndex As Integer) As ClassInformation
        Return Me._classArray(Me._instanceArray.Item(instanceIndex).ClassIndex)
    End Function

    ''' <summary>
    ''' コレクション判定
    ''' </summary>
    ''' <param name="o">判定対象オブジェクト</param>
    ''' <param name="array">コレクション</param>
    ''' <returns>true/false=コレクション/以外</returns>
    Private Shared Function IsCollection(ByVal o As Object, Optional ByRef array As Object() = Nothing) As Boolean
        If Not IsNothing(o) Then
            If IsArray(o) Then
                If Not IsNothing(array) Then array = ToArray(o)
                Return True
            ElseIf TypeOf o Is ArrayList Then
                If Not IsNothing(array) Then array = CType(o, ArrayList).ToArray()
                Return True
            Else
                Dim t As Type = o.GetType()
                If t.IsGenericType AndAlso t.GetGenericTypeDefinition() = GetType(List(Of )) Then
                    If Not IsNothing(array) Then array = ToArray(o)
                    Return True
                End If
            End If
        End If
        Return False
    End Function

    ''' <summary>
    ''' 任意のジェネリック型を動的に生成する
    ''' </summary>
    ''' <param name="genericType">ジェネリック型を構築する元になるジェネリック型定義</param>
    ''' <param name="genericArgMetadata">ジェネリック型に必要な引数メタデータ</param>
    ''' <returns>任意のジェネリック型インスタンス</returns>
    ''' <remarks>genericType:List{}やDictionary{,}など</remarks>
    Public Shared Function DynamicCreateGenericType(ByVal genericType As Type, ByVal genericArgMetadata() As Type) As Object
        Dim dg As Object = Nothing
        If genericType.IsGenericType AndAlso genericType = genericType.GetGenericTypeDefinition() Then
            Dim genericTypeArgumentCount = genericType.GetGenericArguments().Length
            If genericTypeArgumentCount = genericArgMetadata.Length Then
                Dim gtd = genericType.GetGenericTypeDefinition()
                Dim dgtype As Type = gtd.MakeGenericType(genericArgMetadata)
                dg = Activator.CreateInstance(dgtype)
            End If
        End If
        Return dg
    End Function

    ''' <summary>
    ''' 配列を作成する
    ''' </summary>
    ''' <param name="t">型</param>
    ''' <returns></returns>
    Public Shared Function CreateArray(ByVal t As Type) As Object
        Dim typargs As New List(Of Type)
        For Each e As Type In t.GetGenericArguments()
            typargs.Add(e)
        Next
        Dim ret As Object = DynamicCreateGenericType(t, typargs.ToArray())
        If IsNothing(ret) Then
            ret = Activator.CreateInstance(t)
        End If
        Return ret
    End Function

    ''' <summary>
    ''' 配列化
    ''' </summary>
    ''' <param name="o">オブジェクト</param>
    ''' <returns>配列</returns>
    ''' <remarks></remarks>
    Public Shared Function ToArray(ByVal o As Object) As Object()
        Dim ls As New ArrayList()
        Try
            If Not IsNothing(o) Then
                For Each w As Object In o
                    ls.Add(w)
                Next
            End If
        Catch ex As Exception
            System.Diagnostics.Debug.Print("ToArray: error " + ex.ToString())
        End Try
        Return ls.ToArray()
    End Function

    ''' <summary>
    ''' 各種情報クリア
    ''' </summary>
    Private Sub ClearInformations()
        Me._classArray.Clear()
        Me._instanceArray.Clear()
        Me._fieldNameArray.Clear()
        Me._classIndexMap.Clear()
        Me._instanceIndexMap.Clear()
        Me._fieldNameIndexMap.Clear()
        GetClassIndex(GetType(List(Of String)))
        GetClassIndex(GetType(List(Of Integer)))
    End Sub

    ''' <summary>
    ''' Bysonブロック作成
    ''' </summary>
    ''' <param name="o">オブジェクト</param>
    ''' <param name="blocks">ブロック配列</param>
    Private Sub MakeBlock(ByVal o As Object, ByVal blocks As List(Of ByBlock))
        If Not IsNothing(o) AndAlso Not Me._instanceIndexMap.ContainsKey(o) Then
            Dim t As Type = o.GetType()
            If IsDataContractClass(t) Then
                Dim index As Integer = GetInstanceIndex(o)
                If index <> INVALID_INDEX Then
                    blocks.Add(TagBlock.CreateHeadTag(index))
                    Dim map As New HashSet(Of Object)
                    For Each fi As System.Reflection.FieldInfo In GetClassInformationByInstanceIndex(index).Fields
                        Dim fo As Object = fi.GetValue(o)
                        If Not IsNothing(fo) Then
                            GetClassIndex(fo.GetType())
                        End If
                        blocks.Add(TagBlock.CreateTag(GetFieldNameIndex(fi.Name), fo))
                        blocks.Item(blocks.Count - 1).CollectReferenceObject(map)
                    Next
                    blocks.Add(TagBlock.CreateTailTag())
                    For Each e As Object In map
                        MakeBlock(e, blocks)
                    Next
                End If
            End If
        End If
    End Sub

    ''' <summary>
    ''' DataContractクラスか否かを調べる
    ''' </summary>
    ''' <param name="t">クラス</param>
    ''' <returns>true/false=DataContractクラス/以外</returns>
    Public Shared Function IsDataContractClass(ByVal t As Type) As Boolean
        For Each att As Attribute In t.GetCustomAttributes(False)
            Dim contract As System.Runtime.Serialization.DataContractAttribute = TryCast(att, System.Runtime.Serialization.DataContractAttribute)
            If Not IsNothing(contract) Then
                Return True
            End If
        Next
        Return False
    End Function

    ''' <summary>
    ''' 制御文字を取り除く
    ''' </summary>
    ''' <param name="v">文字列</param>
    ''' <returns>処置後の文字列</returns>
    Public Shared Function Trim(ByVal v As String) As String
        If IsNothing(v) Then Return v
        Return v.Trim(New Char() {Chr(0), Chr(8), Chr(9), Chr(10), Chr(13), Chr(10)})
    End Function

    ''' <summary>
    ''' 書込
    ''' </summary>
    ''' <param name="stream">ストリーム</param>
    ''' <param name="o">オブジェクト</param>
    Public Sub Write(ByVal stream As System.IO.Stream, ByVal o As Object, ByVal magic As Long)
        ' 各種属性クリア
        ClearInformations()
        ' ブロック作成
        Dim array As New List(Of ByBlock)
        MakeBlock(o, array)

        ' マジック書込
        PutMagic(stream, magic)
        ' BysonMap保存
        TagBlock.CreateHeadTag(INVALID_INDEX).Write(stream, Me)
        Dim classes As New List(Of String)
        For Each ci As ClassInformation In Me._classArray
            classes.Add(ci.Name)
        Next
        TagBlock.CreateTag(0, classes).Write(stream, Me)
        Dim instances As New List(Of Integer)
        For Each ii As InstanceInformation In Me._instanceArray
            instances.Add(ii.ClassIndex)
        Next
        TagBlock.CreateTag(1, instances).Write(stream, Me)
        TagBlock.CreateTag(2, Me._fieldNameArray).Write(stream, Me)
        TagBlock.CreateTailTag().Write(stream, Me)
        ' ブロック保存
        For Each block As ByBlock In array
            block.Write(stream, Me)
        Next
        ' ファイル終端保存
        TagBlock.CreateEofTag().Write(stream, Me)
    End Sub

    ''' <summary>
    ''' 読込
    ''' </summary>
    ''' <param name="stream">ストリーム</param>
    ''' <returns>オブジェクト</returns>
    Public Function Read(ByVal stream As System.IO.Stream) As Object
        Dim ret As Object = Nothing
        Dim tag As TagBlock
        ClearInformations()
        GetMagic(stream)
        Do While True
            tag = TagBlock.CreateEmptyTag()
            tag.Read(stream, Me)
            If tag.IsHeadTag() Then
                '
            ElseIf tag.IsTailTag() Then
                Exit Do
            ElseIf tag.TagNo = 0 Then
                Me._classArray.Clear()
                Me._classIndexMap.Clear()
                For Each name As String In tag.ResolveValue()
                    Dim ci As New ClassInformation()
                    ci.Name = name
                    ci.ClassType = NameToType(name)
                    ci.Fields = GetFieldInformations(ci.ClassType, ci.FieldMap)
                    Me._classArray.Add(ci)
                    Me._classIndexMap.Add(name, Me._classArray.Count - 1)
                Next
            ElseIf tag.TagNo = 1 Then
                Me._instanceArray.Clear()
                Me._instanceIndexMap.Clear()
                For Each index As Integer In tag.ResolveValue()
                    Dim ii As New InstanceInformation()
                    ii.ClassIndex = index
                    Dim o As Object = System.Activator.CreateInstance(Me._classArray(index).ClassType, New Object() {})
                    ii.Instance = o
                    Me._instanceArray.Add(ii)
                    Me._instanceIndexMap.Add(o, Me._instanceArray.Count - 1)
                Next
            ElseIf tag.TagNo = 2 Then
                Me._fieldNameArray = tag.ResolveValue()
            Else
                Exit Do
            End If
        Loop

        Dim isTop = True
        Dim instanceIndex As Integer = INVALID_INDEX
        Do While True
            tag = TagBlock.CreateEmptyTag()
            tag.Read(stream, Me)
            Dim o As Object = tag.ResolveValue()
            If tag.IsEofTag() Then
                Exit Do
            ElseIf tag.IsHeadTag() Then
                instanceIndex = CType(o, Integer)
                If isTop Then
                    ret = Me._instanceArray(instanceIndex).Instance
                    isTop = False
                End If
            ElseIf tag.IsTailTag() Then
                instanceIndex = INVALID_INDEX
            ElseIf Not tag.IsTailTag() Then
                If instanceIndex <> INVALID_INDEX Then
                    Dim ii As InstanceInformation = Me._instanceArray(instanceIndex)
                    If Not IsNothing(ii) AndAlso ii.ClassIndex <> INVALID_INDEX Then
                        Dim ci As ClassInformation = GetClassInformation(ii.ClassIndex)
                        Dim index As Integer
                        Dim key As String = Trim(Me._fieldNameArray(tag.TagNo))
                        If ci.FieldMap.TryGetValue(key, index) Then
                            Dim tt As Type = ci.Fields(index).FieldType
                            Try
                                If tt.IsSubclassOf(GetType([Enum])) Then
                                    o = System.Convert.ChangeType([Enum].ToObject(tt, o), tt)
                                End If
                                ci.Fields(index).SetValue(ii.Instance, o)
                            Catch ex As Exception
                                System.Diagnostics.Debug.Print("exception:" + ex.ToString())
                                System.Diagnostics.Debug.Print(ex.StackTrace())
                            End Try
                        End If
                    End If
                End If
            End If
        Loop
        Return ret
    End Function

    ''' <summary>
    ''' 名前からクラスを取得
    ''' </summary>
    ''' <param name="name">名前</param>
    ''' <returns>クラス</returns>
    Private Function NameToType(ByVal name As String) As Type
        Dim ret As Type = Type.GetType(name)
        If IsNothing(ret) Then
            ret = Me._asm.GetType(name)
            If IsNothing(ret) Then
                If GetType(System.Drawing.Color).FullName.Equals(name) Then
                    ret = GetType(System.Drawing.Color)
                ElseIf GetType(System.Drawing.Color()).FullName.Equals(name) Then
                    ret = GetType(System.Drawing.Color())
                ElseIf GetType(System.Drawing.Point).FullName.Equals(name) Then
                    ret = GetType(System.Drawing.Point)
                ElseIf GetType(System.Drawing.Point()).FullName.Equals(name) Then
                    ret = GetType(System.Drawing.Point())
                End If
            End If
        End If
        Return ret
    End Function

    ''' <summary>
    ''' マジック書き込み
    ''' </summary>
    ''' <param name="stream">ストリーム</param>
    Private Shared Sub PutMagic(ByVal stream As System.IO.Stream, ByVal magic As Long)
        ByBlock.WriteLong(stream, magic)
    End Sub

    ''' <summary>
    ''' マジック読み込み
    ''' </summary>
    ''' <param name="stream">ストリーム</param>
    ''' <returns>マジック</returns>
    Public Shared Function GetMagic(ByVal stream As System.IO.Stream) As Long
        Return ByBlock.ReadLong(stream)
    End Function

End Class
