﻿''' <summary>
''' タグブロック
''' </summary>
''' <remarks></remarks>
Public Class TagBlock
    Inherits ByBlock

    ''' <summary>
    ''' 特殊タグ番号
    ''' </summary>
    ''' <remarks></remarks>
    Private Const TAG_HEAD = -1     ' 先頭タグ
    Private Const TAG_TAIL = -2     ' 末尾タグ
    Private Const TAG_EOF = -3      ' 終端タグ

    ''' <summary>
    ''' 構築
    ''' </summary>
    ''' <param name="no">タグ番号</param>
    ''' <param name="value">値</param>
    Private Sub New(ByVal no As Integer, ByVal value As Object)
        Me._no = no
        Me._value = New ValueBlock(value)
    End Sub

    ''' <summary>
    ''' 構築
    ''' </summary>
    ''' <param name="no">タグ番号</param>
    Private Sub New(ByVal no As Integer)
        Me._no = no
        Me._value = Nothing
    End Sub

    ''' <summary>
    ''' タグ番号
    ''' </summary>
    Private _no As Integer
    Public ReadOnly Property TagNo As Integer
        Get
            Return Me._no
        End Get
    End Property

    ''' <summary>
    ''' 値ブロック
    ''' </summary>
    Private _value As ValueBlock
    Public ReadOnly Property Value As ValueBlock
        Get
            Return _value
        End Get
    End Property

    ''' <summary>
    ''' 先頭タグ
    ''' </summary>
    ''' <param name="instanceIndex">インスタンスインデックス</param>
    ''' <returns>先頭タグ</returns>
    Public Shared Function CreateHeadTag(ByVal instanceIndex As Integer) As TagBlock
        Return New TagBlock(TAG_HEAD, instanceIndex)
    End Function

    ''' <summary>
    ''' 末尾タグ
    ''' </summary>
    ''' <returns>末尾タグ</returns>
    Public Shared Function CreateTailTag() As TagBlock
        Return New TagBlock(TAG_TAIL)
    End Function

    ''' <summary>
    ''' 終端タグ
    ''' </summary>
    ''' <returns>終端タグ</returns>
    Public Shared Function CreateEofTag() As TagBlock
        Return New TagBlock(TAG_EOF)
    End Function

    ''' <summary>
    ''' タグ生成
    ''' </summary>
    ''' <param name="no">タグ番号</param>
    ''' <param name="value">値</param>
    ''' <returns>タグ</returns>
    Public Shared Function CreateTag(ByVal no As Integer, ByVal value As Object)
        Return New TagBlock(no, value)
    End Function

    ''' <summary>
    ''' 空きタグ生成
    ''' </summary>
    ''' <returns>タグ</returns>
    Public Shared Function CreateEmptyTag()
        Return New TagBlock(0)
    End Function

    ''' <summary>
    ''' 先頭タグ判定
    ''' </summary>
    ''' <returns>true/false=先頭タグ/以外</returns>
    ''' <remarks></remarks>
    Public Function IsHeadTag() As Boolean
        Return TagNo = TAG_HEAD
    End Function

    ''' <summary>
    ''' 末尾タグ判定
    ''' </summary>
    ''' <returns>true/false=末尾タグ/以外</returns>
    ''' <remarks></remarks>
    Public Function IsTailTag() As Boolean
        Return TagNo = TAG_TAIL
    End Function

    ''' <summary>
    ''' 終端タグ判定
    ''' </summary>
    ''' <returns>true/false=終端タグ/以外</returns>
    ''' <remarks></remarks>
    Public Function IsEofTag() As Boolean
        Return TagNo = TAG_EOF
    End Function

    ''' <summary>
    ''' 書込
    ''' </summary>
    ''' <param name="stream">ストリーム</param>
    ''' <param name="map">Bysonマップ</param>
    Public Overrides Sub Write(ByVal stream As System.IO.Stream, ByVal map As BysonMap)
        WriteInteger(stream, Me.TagNo)
        If Not IsNothing(Me.Value) Then
            Me.Value.Write(stream, map)
        End If
    End Sub

    ''' <summary>
    ''' 読込
    ''' </summary>
    ''' <param name="stream">ストリーム</param>
    ''' <param name="map">Bysonマップ</param>
    Public Overrides Sub Read(ByVal stream As System.IO.Stream, ByVal map As BysonMap)
        Me._no = ReadInteger(stream)
        If (Me._no <> TAG_TAIL) Then
            Me._value = New ValueBlock()
            Me._value.Read(stream, map)
        Else
            Me._value = Nothing
        End If
    End Sub

    ''' <summary>
    ''' 参照オブジェクト収集
    ''' </summary>
    ''' <param name="map">参照オブジェクトMAP</param>
    ''' <returns>参照オブジェクトMAP</returns>
    Public Overrides Function CollectReferenceObject(ByVal map As HashSet(Of Object))
        Return If(Not IsNothing(Me.Value), Me.Value.CollectReferenceObject(map), map)
    End Function

    ''' <summary>
    ''' 値を格納する
    ''' </summary>
    ''' <returns>インスタンス</returns>
    Public Overrides Function ResolveValue() As Object
        Dim ret As Object = Nothing
        If Not IsNothing(Me.Value) Then
            ret = Me.Value.ResolveValue()
        End If
        Return ret
    End Function

End Class
