﻿''' <summary>
''' 値ブロック
''' </summary>
''' <remarks></remarks>
Public Class ValueBlock
    Inherits ByBlock

    ''' <summary>
    ''' 値型
    ''' </summary>
    Public Enum ValueType As Short
        NullValue = 0       ' NULL [0byte]
        ByteValue = 1       ' バイト [1byte]
        CharValue = 2       ' 文字コード(Unicode) [2byte]
        ShortValue = 3      ' 短整数 [2byte]
        IntegerValue = 4    ' 整数 [4byte]
        LongValue = 5       ' 長整数 [8byte]
        SingleValue = 6     ' 単精度浮動小数点 [4byte]
        DoubleValue = 7     ' 倍精度浮動小数点 [8byte]
        StringValue = 8     ' 文字列ブロック(バイト文字列長(4)＋文字列) [(4+N*2)byte/N:文字長]
        ObjectValue = 9     ' オブジェクトブロック(インスタンスインデックス) [4byte]
        ListValue = 10      ' List配列ブロック(配列長(4)＋配列要素のブロック型(2)＋ブロックデータ列) [(6+ΣXn)byte/Xn:要素nのブロックデータサイズ]
        EnumValue = 11      ' 列挙型(整数) [4byte]
        DateValue = 12      ' 日付型(長整数) [8byte]
        BoolValue = 13      ' 真偽(短整数) [2byte]
        DictionaryValue = 14    ' 辞書型
        PointValue = 15     ' 座標
        ColorValue = 16     ' 色
        ArrayValue = 17     ' 配列
        ArrayListValue = 18 ' 配列
        ExternalValue = 99  ' 外部型
    End Enum

    ''' <summary>
    ''' 型情報
    ''' </summary>
    Private Structure ValueInfomation
        Public Sub New(ByVal type As ValueType, ByVal body As Type, ByVal typical As Type)
            Me.type = type
            Me.body = body
            Me.typical = typical
        End Sub
        Public type As ValueType    ' 値型
        Public body As Type         ' 値本体クラス
        Public typical As Type      ' 代表型
    End Structure

    ''' <summary>
    ''' 型情報配列
    ''' </summary>
    ''' <remarks></remarks>
    Private Shared ValueInfomations As ValueInfomation() = {
        New ValueInfomation(ValueType.NullValue, Nothing, Nothing),
        New ValueInfomation(ValueType.ByteValue, GetType(ByteValueBody), GetType(Byte)),
        New ValueInfomation(ValueType.CharValue, GetType(CharValueBody), GetType(Char)),
        New ValueInfomation(ValueType.ShortValue, GetType(ShortValueBody), GetType(Short)),
        New ValueInfomation(ValueType.IntegerValue, GetType(IntegerValueBody), GetType(Integer)),
        New ValueInfomation(ValueType.LongValue, GetType(LongValueBody), GetType(Long)),
        New ValueInfomation(ValueType.SingleValue, GetType(SingleValueBody), GetType(Single)),
        New ValueInfomation(ValueType.DoubleValue, GetType(DoubleValueBody), GetType(Double)),
        New ValueInfomation(ValueType.StringValue, GetType(StringValueBody), GetType(String)),
        New ValueInfomation(ValueType.ListValue, GetType(ListValueBody), GetType(List(Of ))),
        New ValueInfomation(ValueType.EnumValue, GetType(EnumValueBody), GetType([Enum])),
        New ValueInfomation(ValueType.DateValue, GetType(DateValueBody), GetType(Date)),
        New ValueInfomation(ValueType.BoolValue, GetType(BoolValueBody), GetType(Boolean)),
        New ValueInfomation(ValueType.DictionaryValue, GetType(DictionaryValueBody), GetType(Dictionary(Of ,))),
        New ValueInfomation(ValueType.PointValue, GetType(PointValueBody), GetType(System.Drawing.Point)),
        New ValueInfomation(ValueType.ColorValue, GetType(ColorValueBody), GetType(System.Drawing.Color)),
        New ValueInfomation(ValueType.ArrayListValue, GetType(ArrayListValueBody), GetType(ArrayList)),
        New ValueInfomation(ValueType.ArrayValue, GetType(ArrayValueBody), Nothing),
        New ValueInfomation(ValueType.ObjectValue, GetType(ObjectValueBody), Nothing),
        New ValueInfomation(ValueType.ExternalValue, GetType(ExternalValueBody), Nothing)
    }

    ''' <summary>
    ''' 構築
    ''' </summary>
    Public Sub New()
        Me._type = ValueType.NullValue
    End Sub

    ''' <summary>
    ''' 構築
    ''' </summary>
    ''' <param name="value">値</param>
    Public Sub New(ByVal value As Object)
        Me._type = ToType(value)
        Me._body = CreateBody(Me._type, value)
    End Sub

    ''' <summary>
    ''' 値型
    ''' </summary>
    Private _type As ValueType
    Public ReadOnly Property Type As ValueType
        Get
            Return Me._type
        End Get
    End Property

    ''' <summary>
    ''' 値
    ''' </summary>
    Private _body As ValueBody = Nothing
    Public ReadOnly Property Body As ValueBody
        Get
            Return Me._body
        End Get
    End Property

    ''' <summary>
    ''' 値生成
    ''' </summary>
    ''' <param name="type">値型</param>
    ''' <param name="value">値</param>
    ''' <returns>値ブロック</returns>
    Private Shared Function CreateBody(ByVal type As ValueType, ByVal value As Object) As ValueBody
        Dim ret As ValueBody = Nothing
        For Each info As ValueInfomation In ValueInfomations
            If (info.type.Equals(type)) Then
                If Not IsNothing(info.body) Then
                    'ret = System.Activator.CreateInstance(info.body, New Object() {value})
                    Try
                        ret = System.Activator.CreateInstance(info.body, New Object() {value})
                    Catch ex As Exception
                        System.Diagnostics.Debug.Print(ex.ToString())
                        System.Diagnostics.Debug.Print(ex.StackTrace())
                        Throw ex
                    End Try
                End If
                Exit For
            End If
        Next
        Return ret
    End Function

    ''' <summary>
    ''' 型判定
    ''' </summary>
    ''' <param name="value">オブジェクト</param>
    ''' <returns>値型</returns>
    Private Shared Function ToType(ByVal value As Object)
        Dim ret As ValueType = ValueType.NullValue
        If (Not IsNothing(value)) Then
            Dim bFind = False
            Dim t As Type = value.GetType()
            For Each info As ValueInfomation In ValueInfomations
                If Not IsNothing(info.typical) AndAlso (t.IsSubclassOf(info.typical) OrElse t.Equals(info.typical)) Then
                    ret = info.type
                    bFind = True
                    Exit For
                ElseIf Not IsNothing(info.typical) AndAlso t.IsGenericType() AndAlso info.typical.IsGenericType() Then
                    Try
                        If t Is info.typical.MakeGenericType(t.GetGenericArguments()) Then
                            ret = info.type
                            bFind = True
                            Exit For
                        End If
                    Catch ex As Exception
                        ' nop
                        System.Diagnostics.Debug.Print("")
                    End Try
                End If
            Next
            If Not bFind Then
                If t.IsArray() AndAlso t.GetArrayRank() = 1 Then
                    ret = ValueType.ArrayValue
                ElseIf BysonMap.IsDataContractClass(t) Then
                    ret = ValueType.ObjectValue
                Else
                    System.Diagnostics.Debug.Print(">>>>>>>>" + t.ToString())
                    ret = ValueType.ExternalValue
                End If
            End If
        End If
        Return ret
    End Function

    ''' <summary>
    ''' 書込
    ''' </summary>
    ''' <param name="stream">ストリーム</param>
    ''' <param name="map">Bysonマップ</param>
    Public Overrides Sub Write(ByVal stream As System.IO.Stream, ByVal map As BysonMap)
        WriteShort(stream, Me.Type)
        If (Not IsNothing(Me.Body)) Then
            Me.Body.Write(stream, map)
        End If
    End Sub

    ''' <summary>
    ''' 読込
    ''' </summary>
    ''' <param name="stream">ストリーム</param>
    ''' <param name="map">Bysonマップ</param>
    Public Overrides Sub Read(ByVal stream As System.IO.Stream, ByVal map As BysonMap)
        Me._type = ReadShort(stream)
        Me._body = CreateBody(Me._type, Nothing)
        If (Not IsNothing(Me.Body)) Then
            Me.Body.Read(stream, map)
        End If
    End Sub

    ''' <summary>
    ''' 参照オブジェクト収集
    ''' </summary>
    ''' <param name="map">参照オブジェクトMAP</param>
    ''' <returns>参照オブジェクトMAP</returns>
    Public Overrides Function CollectReferenceObject(ByVal map As HashSet(Of Object))
        Return If(Not IsNothing(Me.Body), Me.Body.CollectReferenceObject(map), map)
    End Function

    ''' <summary>
    ''' 値を格納する
    ''' </summary>
    ''' <returns>インスタンス</returns>
    Public Overrides Function ResolveValue() As Object
        Dim ret As Object = Nothing
        If Not IsNothing(Me.Body) Then
            ret = Me.Body.ResolveValue()
        End If
        Return ret
    End Function

End Class
