﻿''' <summary>
''' ディクショナリ値
''' </summary>
Public Class DictionaryValueBody
    Inherits ValueBody

    ''' <summary>
    ''' 構築
    ''' <param name="o">オブジェクト</param>
    ''' </summary>
    Public Sub New(ByVal o As Object)
        MyBase.New(ToPairs(o))
        If Not IsNothing(o) Then Me._dictionaryType = o.GetType()
    End Sub

    ''' <summary>
    ''' 型
    ''' </summary>
    Private _dictionaryType As Type = Nothing
    Public ReadOnly Property DictionaryType As Type
        Get
            Return Me._dictionaryType
        End Get
    End Property

    ''' <summary>
    ''' 配列取得
    ''' </summary>
    ''' <returns>配列</returns>
    Public Function GetValue() As List(Of Pair)
        Return Me.Value
    End Function

    ''' <summary>
    ''' 書込
    ''' </summary>
    ''' <param name="stream">ストリーム</param>
    ''' <param name="map">Bysonマップ</param>
    Public Overrides Sub Write(ByVal stream As System.IO.Stream, ByVal map As BysonMap)
        WriteInteger(stream, map.GetClassIndex(Me.DictionaryType))
        WriteInteger(stream, GetValue().Count)
        For Each pair As Pair In GetValue()
            pair.key.Write(stream, map)
        Next
        For Each pair As Pair In GetValue()
            pair.value.Write(stream, map)
        Next
    End Sub

    ''' <summary>
    ''' 読込
    ''' </summary>
    ''' <param name="stream">ストリーム</param>
    ''' <param name="map">Bysonマップ</param>
    Public Overrides Sub Read(ByVal stream As System.IO.Stream, ByVal map As BysonMap)
        GetValue().Clear()
        Me._dictionaryType = map.GetClass(ReadInteger(stream))
        Dim leng As Integer = ReadInteger(stream)
        Dim keys As New List(Of ValueBlock)
        For i As Integer = 0 To leng - 1
            Dim block As ValueBlock = New ValueBlock()
            block.Read(stream, map)
            keys.Add(block)
        Next
        Dim values As New List(Of ValueBlock)
        For i As Integer = 0 To leng - 1
            Dim block As ValueBlock = New ValueBlock()
            block.Read(stream, map)
            values.Add(block)
        Next
        For i As Integer = 0 To keys.Count - 1
            Dim p As New Pair()
            p.key = keys.Item(i)
            p.value = values.Item(i)
            GetValue().Add(p)
        Next
    End Sub

    ''' <summary>
    ''' 参照オブジェクト収集
    ''' </summary>
    ''' <param name="map">参照オブジェクトMAP</param>
    ''' <returns>参照オブジェクトMAP</returns>
    Public Overrides Function CollectReferenceObject(ByVal map As HashSet(Of Object))
        For Each pair As Pair In GetValue()
            map = pair.key.CollectReferenceObject(map)
            map = pair.value.CollectReferenceObject(map)
        Next
        Return map
    End Function

    ''' <summary>
    ''' 値を格納する
    ''' </summary>
    ''' <returns>インスタンス</returns>
    Public Overrides Function ResolveValue() As Object
        Dim ret As Object = Nothing
        If Not IsNothing(Me._dictionaryType) Then
            Dim dg As Object = Activator.CreateInstance(Me._dictionaryType)
            If Not IsNothing(dg) Then
                Dim mes As System.Reflection.MethodInfo = Me._dictionaryType.GetMethod("Add")
                If Not IsNothing(mes) Then
                    For Each pair As Pair In GetValue()
                        mes.Invoke(dg, New Object() {pair.key.ResolveValue(), pair.value.ResolveValue()})
                    Next
                End If
                ret = dg
            End If
        End If
        Return ret
    End Function

End Class
