﻿''' <summary>
''' List値
''' </summary>
Public Class ListValueBody
    Inherits ValueBody

    ''' <summary>
    ''' 構築
    ''' <param name="o">オブジェクト</param>
    ''' </summary>
    Public Sub New(ByVal o As Object)
        MyBase.New(ToBlocks(BysonMap.ToArray(o)))
        If Not IsNothing(o) Then Me._arrayType = o.GetType()
    End Sub

    ''' <summary>
    ''' 配列型
    ''' </summary>
    Private _arrayType As Type = Nothing
    Public ReadOnly Property ArrayType As Type
        Get
            Return Me._arrayType
        End Get
    End Property

    ''' <summary>
    ''' 配列取得
    ''' </summary>
    ''' <returns>配列</returns>
    Public Function GetValue() As List(Of ValueBlock)
        Return Me.Value
    End Function

    ''' <summary>
    ''' 書込
    ''' </summary>
    ''' <param name="stream">ストリーム</param>
    ''' <param name="map">Bysonマップ</param>
    Public Overrides Sub Write(ByVal stream As System.IO.Stream, ByVal map As BysonMap)
        WriteInteger(stream, map.GetClassIndex(Me.ArrayType))
        WriteInteger(stream, GetValue().Count)
        For Each block As ValueBlock In GetValue()
            block.Write(stream, map)
        Next
    End Sub

    ''' <summary>
    ''' 読込
    ''' </summary>
    ''' <param name="stream">ストリーム</param>
    ''' <param name="map">Bysonマップ</param>
    Public Overrides Sub Read(ByVal stream As System.IO.Stream, ByVal map As BysonMap)
        GetValue().Clear()
        Me._arrayType = map.GetClass(ReadInteger(stream))
        Dim leng As Integer = ReadInteger(stream)
        For i As Integer = 0 To leng - 1
            Dim block As ValueBlock = New ValueBlock()
            block.Read(stream, map)
            GetValue().Add(block)
        Next
    End Sub

    ''' <summary>
    ''' 参照オブジェクト収集
    ''' </summary>
    ''' <param name="map">参照オブジェクトMAP</param>
    ''' <returns>参照オブジェクトMAP</returns>
    Public Overrides Function CollectReferenceObject(ByVal map As HashSet(Of Object))
        For Each block As ValueBlock In GetValue()
            map = block.CollectReferenceObject(map)
        Next
        Return map
    End Function

    ''' <summary>
    ''' 値を格納する
    ''' </summary>
    ''' <returns>インスタンス</returns>
    Public Overrides Function ResolveValue() As Object
        Dim ret As Object = Nothing
        If Not IsNothing(Me._arrayType) Then
            Dim dg As Object = BysonMap.CreateArray(Me._arrayType)
            If Not IsNothing(dg) Then
                Dim mes As System.Reflection.MethodInfo = Me._arrayType.GetMethod("Add")
                If Not IsNothing(mes) Then
                    For Each bl As ValueBlock In GetValue()
                        mes.Invoke(dg, New Object() {bl.ResolveValue()})
                    Next
                End If
                ret = dg
            End If
        End If
        Return ret
    End Function

End Class
