﻿''' <summary>
''' Byson保管
''' </summary>
''' <remarks>Copyright System Concept Ltd. 2013 All rights reserved</remarks>
Public Class BysonStorage
    Implements ExternalBlockListener

    ' デシリアライズ中フラグ(true:デシリアライズ中)
    Private _isDeserializing As Boolean = False
    ' シリアライズ中フラグ(true:シリアライズ中)
    Private _isSerializing As Boolean = False
    ' マジックNo
    Private _magicNo As Long = 0L
    ' アセンブリ
    Private _asm As System.Reflection.Assembly

    ''' <summary>
    ''' 構築
    ''' </summary>
    ''' <param name="magicNo">マジックNo</param>
    ''' <remarks></remarks>
    Public Sub New(ByVal magicNo As Long, ByVal asm As System.Reflection.Assembly)
        Me._magicNo = magicNo
        Me._asm = asm
    End Sub

    ''' <summary>
    ''' ストレージ中か否かを調べる
    ''' </summary>
    ''' <returns>true:ストレージ中</returns>
    Public Function IsStoraging() As Boolean
        Return IsDeserializing() OrElse IsSerializing()
    End Function

    ''' <summary>
    ''' デシリアライズ中か否かを調べる
    ''' </summary>
    ''' <returns>true:デシリアライズ中</returns>
    Private Function IsDeserializing() As Boolean
        Return Me._isDeserializing
    End Function

    ''' <summary>
    ''' シリアライズ中か否かを調べる
    ''' </summary>
    ''' <returns>true:シリアライズ中</returns>
    Private Function IsSerializing() As Boolean
        Return Me._isSerializing
    End Function

    ''' <summary>
    ''' 書込
    ''' </summary>
    ''' <param name="stream">ストリーム</param>
    ''' <param name="o">オブジェクト</param>
    Public Sub Write(ByVal stream As System.IO.Stream, ByVal o As Object)
        Try
            Me._isSerializing = True
            Dim bymap As New BysonMap(Me._asm)
            bymap.SetExternalBlockListener(Me)
            bymap.Write(stream, o, Me._magicNo)
        Finally
            Me._isSerializing = False
        End Try
    End Sub

    ''' <summary>
    ''' 書込
    ''' </summary>
    ''' <param name="path">ファイルパス</param>
    ''' <param name="o">オブジェクト</param>
    Public Sub Write(ByVal path As String, ByVal o As Object)
        Dim fs As System.IO.Stream = Nothing
        Try
            fs = System.IO.File.Open(path, System.IO.FileMode.Create, System.IO.FileAccess.Write)
            Write(fs, o)
        Finally
            If Not IsNothing(fs) Then
                fs.Close()
            End If
        End Try
    End Sub

    ''' <summary>
    ''' 読込
    ''' </summary>
    ''' <param name="stream">ストリーム</param>
    ''' <returns>オブジェクト</returns>
    Public Function Read(ByVal stream As System.IO.Stream) As Object
        Try
            Me._isDeserializing = True
            Dim bymap As New BysonMap(Me._asm)
            bymap.SetExternalBlockListener(Me)
            Return bymap.Read(stream)
        Finally
            Me._isDeserializing = False
        End Try
    End Function

    ''' <summary>
    ''' 読込
    ''' </summary>
    ''' <param name="path">ファイルパス</param>
    ''' <returns>オブジェクト</returns>
    ''' <remarks></remarks>
    Public Function Read(ByVal path As String) As Object
        Dim ret As Object = Nothing
        Dim fs As System.IO.Stream = Nothing
        Try
            fs = System.IO.File.Open(path, System.IO.FileMode.Open, System.IO.FileAccess.Read)
            ret = Read(fs)
        Finally
            If Not IsNothing(fs) Then
                fs.Close()
            End If
        End Try
        Return ret
    End Function

    ''' <summary>
    ''' マジックNo比較
    ''' </summary>
    ''' <param name="magicNo">マジックNo</param>
    ''' <returns>true/false=マジックNo一致/不一致</returns>
    ''' <remarks></remarks>
    Protected Overridable Function CompairMagicNo(ByVal magicNo As Long) As Boolean
        Return (magicNo = Me._magicNo)
    End Function

    ''' <summary>
    ''' 自分のストリームか否か
    ''' </summary>
    ''' <param name="stream">ストリーム</param>
    ''' <returns>true/false=自分のストリーム/以外</returns>
    Public Function IsMyStream(ByVal stream As System.IO.Stream) As Boolean
        Return CompairMagicNo(BysonMap.GetMagic(stream))
    End Function

    ''' <summary>
    ''' 自分のファイルか否か
    ''' </summary>
    ''' <param name="path">ファイルパス</param>
    ''' <returns>true/false=自分のファイル/以外</returns>
    Public Function IsMyFile(ByVal path As String) As Boolean
        Dim ret As Boolean = False
        Dim fs As System.IO.Stream = Nothing
        Try
            fs = System.IO.File.Open(path, System.IO.FileMode.Open, System.IO.FileAccess.Read)
            ret = IsMyStream(fs)
        Catch ex As Exception
            '
        Finally
            If Not IsNothing(fs) Then
                fs.Close()
            End If
        End Try
        Return ret
    End Function

    ''' <summary>
    ''' 書込
    ''' </summary>
    ''' <param name="stream">ストリーム</param>
    ''' <param name="map">Bysonマップ</param>
    ''' <param name="o">書込オブジェクト</param>
    Protected Overridable Sub ExternalWrite(ByVal stream As System.IO.Stream, ByVal map As BysonMap, ByVal o As Object) Implements ExternalBlockListener.Write
        Trace.Assert(False)
    End Sub

    ''' <summary>
    ''' 読込
    ''' </summary>
    ''' <param name="stream">ストリーム</param>
    ''' <param name="map">Bysonマップ</param>
    ''' <param name="typename">型名称</param>
    ''' <returns>オブジェクト</returns>
    Protected Overridable Function ExternalRead(ByVal stream As System.IO.Stream, ByVal map As BysonMap, ByVal typename As String) As Object Implements ExternalBlockListener.Read
        Trace.Assert(False)
        Return Nothing
    End Function

End Class
